@@ -3,10 +3,10 @@ package Finance::IIF;
 use 5.006;
 use strict;
 use warnings;
-use Carp;
-use IO::File;
+use Carp qw(carp croak);
+use IO::File ();
 
-our $VERSION = '0.03.01';
+our $VERSION = '0.21_02';
 $VERSION = eval $VERSION;
 
 sub new {
@@ -14,13 +14,16 @@ sub new {
     my %opt   = @_;
     my $self  = {};
 
-    $self->{debug}                   = $opt{debug}                   || 0;
-    $self->{field_separator}         = $opt{field_separator}         || "\t";
-    $self->{input_record_separator}  = $opt{input_record_separator}  || $/;
-    $self->{output_record_separator} = $opt{output_record_separator} || $\;
+    $self->{debug}           = $opt{debug}           || 0;
+    $self->{autodetect}      = $opt{autodetect}      || 0;
+    $self->{field_separator} = $opt{field_separator} || "\t";
 
     bless( $self, $class );
 
+    if ( $opt{record_separator} ) {
+        $self->record_separator( $opt{record_separator} );
+    }
+
     if ( $opt{file} ) {
         $self->file( $opt{file} );
         $self->open;
@@ -42,6 +45,14 @@ sub file {
     }
 }
 
+sub record_separator {
+    my $self = shift;
+    if (@_) {
+        $self->{record_separator} = $_[0] if ( $_[0] );
+    }
+    return $self->{record_separator} || $/;
+}
+
 sub _filehandle {
     my $self = shift;
     if (@_) {
@@ -49,6 +60,7 @@ sub _filehandle {
         $self->{_filehandle} = IO::File->new(@args)
           or croak("Failed to open file '$args[0]': $!");
         $self->{_linecount} = 0;
+        $self->{_filehandle}->binmode();
     }
     if ( !$self->{_filehandle} ) {
         croak("No filehandle available");
@@ -63,6 +75,22 @@ sub open {
     }
     if ( $self->file ) {
         $self->_filehandle( $self->file );
+        if ( $self->{autodetect} ) {
+            if ( $self->_filehandle->seek( -2, 2 ) ) {
+                my $buffer = "";
+                $self->_filehandle->read( $buffer, 2 );
+                if ( $buffer eq "\015\012" ) {
+                    $self->record_separator("\015\012");
+                }
+                elsif ( $buffer =~ /\012$/ ) {
+                    $self->record_separator("\012");
+                }
+                elsif ( $buffer =~ /\015$/ ) {
+                    $self->record_separator("\015");
+                }
+            }
+        }
+        $self->reset();
     }
     else {
         croak("No file specified");
@@ -76,9 +104,26 @@ sub next {
     if ( $self->_filehandle->eof ) {
         return undef;
     }
+    my $line;
     while ( !$self->_filehandle->eof && $continue ) {
-        my $line = $self->_getline;
-        next if ( $line =~ /^\s*$/ );
+
+        # IIF files sometimes contain embedded record_separators so if
+        # the previous line didn't have enough fields keep adding to it
+        if ( $self->{_need_more} ) {
+            if ( $self->{_need_more} > 8 ) {
+                $self->_warning("Giving up trying to get next record");
+                delete $self->{_need_more};
+                $continue = 0;
+            }
+            else {
+                $line .= $self->record_separator . $self->_getline;
+            }
+        }
+        else {
+            $line = $self->_getline;
+            next if ( $line =~ /^\s*$/ );
+        }
+
         my @data = $self->_parseline($line);
 
         if ( $self->{debug} > 1 ) {
@@ -93,7 +138,6 @@ sub next {
             $self->{headerfields} = \@data;
         }
         elsif ( $data[0] eq $self->{header} ) {
-            $continue = 0;
             $object{header} = shift(@data);
             my $num_hdr = scalar( @{ $self->{headerfields} } );
             my $num_dat = scalar(@data);
@@ -101,6 +145,10 @@ sub next {
             # have seen IIF timer data where last column (USEID) was
             # missing but QuickBooks imports the data without error
             if ( $num_dat < ( $num_hdr - 1 ) ) {
+                $self->{_need_more}++;
+            }
+            elsif ( $num_dat > $num_hdr ) {
+                $self->{_need_more} = $continue = 0;
                 no warnings 'uninitialized';
                 $self->_warning( "parse error: found $num_dat fields but"
                       . " expected $num_hdr." );
@@ -116,6 +164,7 @@ sub next {
                 );
             }
             else {
+                $self->{_need_more} = $continue = 0;
                 for ( my $i = 0 ; $i <= $#{ $self->{headerfields} } ; $i++ ) {
                     my $val = defined( $data[$i] ) ? $data[$i] : "";
                     $object{ $self->{headerfields}[$i] } = $val;
@@ -141,27 +190,27 @@ sub _parseline {
     my $sep  = $self->{field_separator} || "\t";
     my @data;
     while ( defined $line ) {
-        if ( $line =~ /^"(.*?)(?:[^\\]["])[$sep](.*)/ ) {
+        if ( $line =~ /^"(.*?)(?:[^\\]["])[$sep](.*)/s ) {
             warn("parse1: data($1) line($2)\n") if ( $self->{debug} > 2 );
             $line = $2;
             push( @data, $1 );
         }
-        elsif ( $line =~ /^([^$sep]+)[$sep](.*)/ ) {
+        elsif ( $line =~ /^([^$sep]+)[$sep](.*)/s ) {
             warn("parse2: data($1) line($2)\n") if ( $self->{debug} > 2 );
             $line = $2;
             push( @data, $1 );
         }
-        elsif ( $line =~ /^[$sep](.*)/ ) {
+        elsif ( $line =~ /^[$sep](.*)/s ) {
             warn("parse3: data() line($1)\n") if ( $self->{debug} > 2 );
             $line = $1;
             push( @data, "" );
         }
-        elsif ( $line =~ /^"(.*?)(?:[^\\]["])$/ ) {
+        elsif ( $line =~ /^"(.*?)(?:[^\\]["])$/s ) {
             warn("parse4: data($1) line()\n") if ( $self->{debug} > 2 );
             $line = undef;
             push( @data, $1 );
         }
-        elsif ( $line =~ /^(.+)$/ ) {
+        elsif ( $line =~ /^(.+)$/s ) {
             warn("parse5: data($1) line()\n") if ( $self->{debug} > 2 );
             $line = undef;
             push( @data, $1 );
@@ -177,10 +226,10 @@ sub _parseline {
 
 sub _getline {
     my $self = shift;
-    local $/ = $self->{input_record_separator};
+    local $/ = $self->record_separator;
     my $line = $self->_filehandle->getline;
-    chomp($line);
     $self->{_linecount}++;
+    chomp($line);
     return $line;
 }
 
@@ -197,7 +246,6 @@ sub _warning {
 sub reset {
     my $self = shift;
     $self->_filehandle->seek( 0, 0 );
-    $self->next;
 }
 
 sub close {
@@ -206,6 +254,7 @@ sub close {
 }
 
 1;
+
 __END__
 
 =head1 NAME
@@ -219,9 +268,9 @@ Finance::IIF - Parse and create IIF files for QuickBooks
   my $iif = Finance::IIF->new( file => "test.iif" );
   
   while ( my $record = $iif->next ) {
-      print( "Header: " . $record->{header} . "\n" );
+      print( "Header: ", $record->{header}, "\n" );
       foreach my $key ( keys %{$record} ) {
-          print( "     " . $key . ": " . $record->{$key} . "\n" );
+          print( "     ", $key, ": ", $record->{$key}, "\n" );
       }
   }
 
@@ -279,7 +328,7 @@ will do any additional processing or validation as required.
           INVMEMO
           SADDR1
           SADDR2
- category                  L                 
+ category                  L
 
 QuickBooks doesn't support investment accounts Quicken does.
 
@@ -587,23 +636,47 @@ If the file is specified it will be opened on new.
 
 =item file
 
-Specifies file to use for processing.  See L</file()> for details.
+Specifies file to use for processing.  See L<file()|/file()> for details.
 
   my $in = Finance::IIF->new( file => "myfile" );
 OR
   my $in = Finance::IIF->new( file => [ "myfile", "<:crlf" ] );
 
-=item input_record_separator
+=item record_separator
 
-Can be used to redefine the IIF input record separator.  Default is $/.
+Can be used to redefine the IIF record separator.  Default is $/.
 
-  my $in = Finance::IIF->new( input_record_separator => "\n" );
+  my $in = Finance::IIF->new( record_separator => "\012" );
 
-=item output_record_separator
+Note: For MacOS X it may be necessary to change this to "\015".  See
+L<autodetect|/autodetect> for another option.
 
-Can be used to redefine the IIF output record separator.  Default is $\.
+=item autodetect
 
-  my $out = Finance::IIF->new( output_record_separator => "\n" );
+Enable auto detection of the record separator based on the file
+contents.  Default is "0".
+
+  my $in = Finance::IIF->new( autodetect => 1 );
+
+Perl uses $/ to define line separators for text files.  Perl sets this
+value according to the OS perl is running on:
+
+  Windows="\015\012"
+  Mac="\015"
+  Unix="\012"
+
+In many cases you may find yourself with text files that do not match
+the OS.  In these cases Finance::IIF by default will not process that
+IIF file correctly. This feature is an attempt to help with the most
+common cases of having the wrong text file for the OS Finance::IIF is
+running on.
+
+This feature depends on being able to seek to the end of the file and
+reading the last 2 characters to determine the proper separator. If a
+seek can not be performed or the last 2 characters are not a proper
+separator the record_separator will default to $/ or the value passed
+in. If a valid record_separator is found then it will be set according
+to what was in the file.
 
 =item debug
 
@@ -618,7 +691,7 @@ Can be used to output debug information.  Default is "0".
 Specify file name and optionally additional parameters that will be
 used to obtain a filehandle.  The argument can be a filename (SCALAR)
 an ARRAY reference or an ARRAY whose values must be valid arguments
-for passing to L<IO::File/new>.
+for passing to IO::File->new.
 
   $iif->file("myfile");
  OR
@@ -626,13 +699,22 @@ for passing to L<IO::File/new>.
  OR
   $iif->file( "myfile", "<:crlf" );
 
+=head2 record_separator()
+
+Returns the currently used record_separator.  This is used primarly in
+situations where you open a IIF file with autodetect and then want to
+write out a IIF file in the same format.
+
+  my $iif = Finance::IIF->new( file => "input.iif", autodetect => 1 );
+  my $rs  = $iif->record_separator;
+
 =head2 open()
 
 Open already specified file.
 
   $iif->open();
 
-Opens specified file using L</file()>.
+Open also supports the same arguments as L<file()|/file()>.
 
   $iif->open("myfile");
 
@@ -677,7 +759,7 @@ Test cases for IIF parsing
 
 =head1 SEE ALSO
 
-L<Carp>, L<IO::File>
+L<Carp|Carp>, L<IO::File|IO::File>
 
 =head1 AUTHORS
 
@@ -685,12 +767,16 @@ Matthew McGillis E<lt>matthew@mcgillis.orgE<gt> L<http://www.mcgillis.org/>
 
 Phil Lobbes E<lt>phil at perkpartners dot comE<gt>
 
-Project maintaned at L<http://www.sourceforge.net/projects/finance-iif>
+Project maintained at L<http://sourceforge.net/projects/finance-iif>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2006 by Matthew McGillis and Phil Lobbes.  All rights
-reserved.  This library is free software; you can redistribute it
-and/or modify it under the same terms as Perl itself.
+Copyright (C) 2006-2007 by Matthew McGillis and Phil Lobbes.  All
+rights reserved.
+
+=head1 LICENSE
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
 
 =cut
